@@ -8,12 +8,13 @@ RCPP_EXPOSED_CLASS(Converter);
 RCPP_MODULE(Module){
   using namespace Rcpp ;
   class_<Converter>("Converter")
-    .constructor<t_positions>("Construct converter.")
+    .constructor<t_positions>("Construct converter (use_names = FALSE).")
+    .constructor<t_positions, bool>("Construct converter.")
     .method("convert", &Converter::convert)
     .method("size", &Converter::size)
     .method("insert_founder", &Converter::insert_founder)
     .method("[[<-", &Converter::insert_founder)
-    .method("check_xodat_individual", &Converter::check_xodat_individual)
+    // .method("check_xodat_individual", &Converter::check_xodat_individual)
     ;
 
   // class_<Converter2>("Converter2")
